@@ -1,6 +1,7 @@
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
+from collections import Iterable
 from itertools import chain
 try:
     from urllib.parse import urlencode
@@ -10,11 +11,9 @@ except:
 from django import forms
 from django.db.models.fields import BLANK_CHOICE_DASH
 from django.forms.widgets import flatatt
-try:
-    from django.utils.encoding import force_text
-except:  # pragma: nocover
-    from django.utils.encoding import force_unicode as force_text  # noqa
+from django.utils.encoding import force_text
 from django.utils.safestring import mark_safe
+from django.utils.six import string_types
 from django.utils.translation import ugettext as _
 
 
@@ -80,7 +79,7 @@ class LinkWidget(forms.Widget):
 
 class RangeWidget(forms.MultiWidget):
     def __init__(self, attrs=None):
-        widgets = (forms.TextInput(attrs=attrs), forms.TextInput(attrs=attrs))
+        widgets = (forms.TextInput, forms.TextInput)
         super(RangeWidget, self).__init__(widgets, attrs)
 
     def decompress(self, value):
@@ -97,3 +96,60 @@ class LookupTypeWidget(forms.MultiWidget):
         if value is None:
             return [None, None]
         return value
+
+
+class BooleanWidget(forms.Select):
+    """Convert true/false values into the internal Python True/False.
+    This can be used for AJAX queries that pass true/false from JavaScript's
+    internal types through.
+    """
+    def __init__(self, attrs=None):
+        choices = (('', _('Unknown')),
+                   ('true', _('Yes')),
+                   ('false', _('No')))
+        super(BooleanWidget, self).__init__(attrs, choices)
+
+    def render(self, name, value, attrs=None):
+        try:
+            value = {
+                True: 'true',
+                False: 'false',
+                '1': 'true',
+                '0': 'false'
+            }[value]
+        except KeyError:
+            value = ''
+        return super(BooleanWidget, self).render(name, value, attrs)
+
+    def value_from_datadict(self, data, files, name):
+        value = data.get(name, None)
+        if isinstance(value, string_types):
+            value = value.lower()
+
+        return {
+            '1': True,
+            '0': False,
+            'true': True,
+            'false': False,
+            True: True,
+            False: False,
+        }.get(value, None)
+
+
+class CSVWidget(forms.TextInput):
+    def _isiterable(self, value):
+        return isinstance(value, Iterable) and not isinstance(value, string_types)
+
+    def value_from_datadict(self, data, files, name):
+        value = super(CSVWidget, self).value_from_datadict(data, files, name)
+
+        if value is not None:
+            return value.split(',')
+        return None
+
+    def render(self, name, value, attrs=None):
+        if self._isiterable(value):
+            value = [force_text(self._format_value(v)) for v in value]
+            value = ','.join(list(value))
+
+        return super(CSVWidget, self).render(name, value, attrs)
