@@ -1,27 +1,23 @@
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
-from datetime import date, datetime, time
+from datetime import date, time, timedelta, datetime
 import mock
-import sys
-
-if sys.version_info >= (2, 7):
-    import unittest
-else:  # pragma: nocover
-    from django.utils import unittest  # noqa
-
-from datetime import timedelta
+import warnings
+import unittest
 
 from django import forms
-from django.test import TestCase
-
+from django.test import TestCase, override_settings
 
+from django_filters import filters
 from django_filters.fields import (
     Lookup,
     RangeField,
     DateRangeField,
+    DateTimeRangeField,
     TimeRangeField,
-    LookupTypeField)
+    LookupTypeField,
+    BaseCSVField)
 from django_filters.filters import (
     Filter,
     CharFilter,
@@ -38,8 +34,13 @@ from django_filters.filters import (
     RangeFilter,
     DateRangeFilter,
     DateFromToRangeFilter,
+    DateTimeFromToRangeFilter,
     TimeRangeFilter,
     AllValuesFilter,
+    BaseCSVFilter,
+    BaseInFilter,
+    BaseRangeFilter,
+    UUIDFilter,
     LOOKUP_TYPES)
 
 from tests.models import Book, User
@@ -49,7 +50,7 @@ class FilterTests(TestCase):
 
     def test_creation(self):
         f = Filter()
-        self.assertEqual(f.lookup_type, 'exact')
+        self.assertEqual(f.lookup_expr, 'exact')
         self.assertEqual(f.exclude, False)
 
     def test_creation_order(self):
@@ -69,26 +70,40 @@ class FilterTests(TestCase):
         self.assertIsInstance(field, forms.Field)
         self.assertEqual(field.help_text, 'This is an exclusion filter')
 
-    def test_field_with_single_lookup_type(self):
-        f = Filter(lookup_type='iexact')
+    @override_settings(FILTERS_HELP_TEXT_FILTER=False)
+    def test_default_field_settings(self):
+        f = Filter()
+        field = f.field
+        self.assertIsInstance(field, forms.Field)
+        self.assertEqual(field.help_text, '')
+
+    @override_settings(FILTERS_HELP_TEXT_EXCLUDE=False)
+    def test_field_with_exclusion_settings(self):
+        f = Filter(exclude=True)
+        field = f.field
+        self.assertIsInstance(field, forms.Field)
+        self.assertEqual(field.help_text, '')
+
+    def test_field_with_single_lookup_expr(self):
+        f = Filter(lookup_expr='iexact')
         field = f.field
         self.assertIsInstance(field, forms.Field)
 
-    def test_field_with_none_lookup_type(self):
-        f = Filter(lookup_type=None)
+    def test_field_with_none_lookup_expr(self):
+        f = Filter(lookup_expr=None)
         field = f.field
         self.assertIsInstance(field, LookupTypeField)
         choice_field = field.fields[1]
         self.assertEqual(len(choice_field.choices), len(LOOKUP_TYPES))
 
-    def test_field_with_lookup_type_and_exlusion(self):
-        f = Filter(lookup_type=None, exclude=True)
+    def test_field_with_lookup_expr_and_exlusion(self):
+        f = Filter(lookup_expr=None, exclude=True)
         field = f.field
         self.assertIsInstance(field, LookupTypeField)
         self.assertEqual(field.help_text, 'This is an exclusion filter')
 
-    def test_field_with_list_lookup_type(self):
-        f = Filter(lookup_type=('istartswith', 'iendswith'))
+    def test_field_with_list_lookup_expr(self):
+        f = Filter(lookup_expr=('istartswith', 'iendswith'))
         field = f.field
         self.assertIsInstance(field, LookupTypeField)
         choice_field = field.fields[1]
@@ -157,22 +172,22 @@ class FilterTests(TestCase):
 
     def test_filtering_with_list_value(self):
         qs = mock.Mock(spec=['filter'])
-        f = Filter(name='somefield', lookup_type=['some_lookup_type'])
-        result = f.filter(qs, Lookup('value', 'some_lookup_type'))
-        qs.filter.assert_called_once_with(somefield__some_lookup_type='value')
+        f = Filter(name='somefield', lookup_expr=['some_lookup_expr'])
+        result = f.filter(qs, Lookup('value', 'some_lookup_expr'))
+        qs.filter.assert_called_once_with(somefield__some_lookup_expr='value')
         self.assertNotEqual(qs, result)
 
     def test_filtering_skipped_with_list_value_with_blank(self):
         qs = mock.Mock()
-        f = Filter(name='somefield', lookup_type=['some_lookup_type'])
-        result = f.filter(qs, Lookup('', 'some_lookup_type'))
+        f = Filter(name='somefield', lookup_expr=['some_lookup_expr'])
+        result = f.filter(qs, Lookup('', 'some_lookup_expr'))
         self.assertListEqual(qs.method_calls, [])
         self.assertEqual(qs, result)
 
     def test_filtering_skipped_with_list_value_with_blank_lookup(self):
-        return # Now field is required to provide valid lookup_type if it provides any
+        return  # Now field is required to provide valid lookup_expr if it provides any
         qs = mock.Mock(spec=['filter'])
-        f = Filter(name='somefield', lookup_type=None)
+        f = Filter(name='somefield', lookup_expr=None)
         result = f.filter(qs, Lookup('value', ''))
         qs.filter.assert_called_once_with(somefield__exact='value')
         self.assertNotEqual(qs, result)
@@ -189,7 +204,44 @@ class FilterTests(TestCase):
         qs = mock.Mock(spec=['filter', 'distinct'])
         f = Filter(name='somefield', distinct=True)
         f.filter(qs, 'value')
-        result = qs.distinct.assert_called_once()
+        result = qs.distinct.assert_called_once_with()
+        self.assertNotEqual(qs, result)
+
+    def test_lookup_type_deprecation(self):
+        """
+        Make sure user is alerted when using deprecated ``lookup_type``.
+        """
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            Filter(lookup_type='exact')
+            self.assertEqual(len(w), 1)
+            self.assertTrue(issubclass(w[-1].category, DeprecationWarning))
+
+
+class CustomFilterWithBooleanCheckTests(TestCase):
+
+    def setUp(self):
+        super(CustomFilterWithBooleanCheckTests, self).setUp()
+
+        class CustomTestFilter(Filter):
+            def filter(self_, qs, value):
+                if not value:
+                    return qs
+                return super(CustomTestFilter, self_).filter(qs, value)
+
+        self.test_filter_class = CustomTestFilter
+
+    def test_lookup_false(self):
+        qs = mock.Mock(spec=['filter'])
+        f = self.test_filter_class(name='somefield')
+        result = f.filter(qs, Lookup('', 'exact'))
+        self.assertEqual(qs, result)
+
+    def test_lookup_true(self):
+        qs = mock.Mock(spec=['filter'])
+        f = self.test_filter_class(name='somefield')
+        result = f.filter(qs, Lookup('somesearch', 'exact'))
+        qs.filter.assert_called_once_with(somefield__exact='somesearch')
         self.assertNotEqual(qs, result)
 
 
@@ -201,6 +253,14 @@ class CharFilterTests(TestCase):
         self.assertIsInstance(field, forms.CharField)
 
 
+class UUIDFilterTests(TestCase):
+
+    def test_default_field(self):
+        f = UUIDFilter()
+        field = f.field
+        self.assertIsInstance(field, forms.UUIDField)
+
+
 class BooleanFilterTests(TestCase):
 
     def test_default_field(self):
@@ -212,14 +272,14 @@ class BooleanFilterTests(TestCase):
         qs = mock.Mock(spec=['filter'])
         f = BooleanFilter(name='somefield')
         result = f.filter(qs, True)
-        qs.filter.assert_called_once_with(somefield=True)
+        qs.filter.assert_called_once_with(somefield__exact=True)
         self.assertNotEqual(qs, result)
 
     def test_filtering_exclude(self):
         qs = mock.Mock(spec=['exclude'])
         f = BooleanFilter(name='somefield', exclude=True)
         result = f.filter(qs, True)
-        qs.exclude.assert_called_once_with(somefield=True)
+        qs.exclude.assert_called_once_with(somefield__exact=True)
         self.assertNotEqual(qs, result)
 
     @unittest.expectedFailure
@@ -237,6 +297,13 @@ class BooleanFilterTests(TestCase):
         self.assertListEqual(qs.method_calls, [])
         self.assertEqual(qs, result)
 
+    def test_filtering_lookup_expr(self):
+        qs = mock.Mock(spec=['filter'])
+        f = BooleanFilter(name='somefield', lookup_expr='isnull')
+        result = f.filter(qs, True)
+        qs.filter.assert_called_once_with(somefield__isnull=True)
+        self.assertNotEqual(qs, result)
+
 
 class ChoiceFilterTests(TestCase):
 
@@ -493,10 +560,10 @@ class NumericRangeFilterTests(TestCase):
         result = f.filter(qs, None)
         self.assertEqual(qs, result)
 
-    def test_field_with_lookup_type(self):
+    def test_field_with_lookup_expr(self):
         qs = mock.Mock()
         value = mock.Mock(start=20, stop=30)
-        f = NumericRangeFilter(lookup_type=('overlap'))
+        f = NumericRangeFilter(lookup_expr=('overlap'))
         f.filter(qs, value)
         qs.filter.assert_called_once_with(None__overlap=(20, 30))
 
@@ -557,10 +624,10 @@ class RangeFilterTests(TestCase):
         result = f.filter(qs, None)
         self.assertEqual(qs, result)
 
-    def test_filtering_ignores_lookup_type(self):
+    def test_filtering_ignores_lookup_expr(self):
         qs = mock.Mock()
         value = mock.Mock(start=20, stop=30)
-        f = RangeFilter(lookup_type='gte')
+        f = RangeFilter(lookup_expr='gte')
         f.filter(qs, value)
         qs.filter.assert_called_once_with(None__range=(20, 30))
 
@@ -690,15 +757,61 @@ class DateFromToRangeFilterTests(TestCase):
         result = f.filter(qs, None)
         self.assertEqual(qs, result)
 
-    def test_filtering_ignores_lookup_type(self):
+    def test_filtering_ignores_lookup_expr(self):
         qs = mock.Mock()
         value = mock.Mock(start=date(2015, 4, 7), stop=date(2015, 9, 6))
-        f = DateFromToRangeFilter(lookup_type='gte')
+        f = DateFromToRangeFilter(lookup_expr='gte')
         f.filter(qs, value)
         qs.filter.assert_called_once_with(
             None__range=(date(2015, 4, 7), date(2015, 9, 6)))
 
 
+class DateTimeFromToRangeFilterTests(TestCase):
+
+    def test_default_field(self):
+        f = DateTimeFromToRangeFilter()
+        field = f.field
+        self.assertIsInstance(field, DateTimeRangeField)
+
+    def test_filtering_range(self):
+        qs = mock.Mock(spec=['filter'])
+        value = mock.Mock(
+            start=datetime(2015, 4, 7, 8, 30), stop=datetime(2015, 9, 6, 11, 45))
+        f = DateTimeFromToRangeFilter()
+        f.filter(qs, value)
+        qs.filter.assert_called_once_with(
+            None__range=(datetime(2015, 4, 7, 8, 30), datetime(2015, 9, 6, 11, 45)))
+
+    def test_filtering_start(self):
+        qs = mock.Mock(spec=['filter'])
+        value = mock.Mock(start=datetime(2015, 4, 7, 8, 30), stop=None)
+        f = DateTimeFromToRangeFilter()
+        f.filter(qs, value)
+        qs.filter.assert_called_once_with(None__gte=datetime(2015, 4, 7, 8, 30))
+
+    def test_filtering_stop(self):
+        qs = mock.Mock(spec=['filter'])
+        value = mock.Mock(start=None, stop=datetime(2015, 9, 6, 11, 45))
+        f = DateTimeFromToRangeFilter()
+        f.filter(qs, value)
+        qs.filter.assert_called_once_with(None__lte=datetime(2015, 9, 6, 11, 45))
+
+    def test_filtering_skipped_with_none_value(self):
+        qs = mock.Mock(spec=['filter'])
+        f = DateTimeFromToRangeFilter()
+        result = f.filter(qs, None)
+        self.assertEqual(qs, result)
+
+    def test_filtering_ignores_lookup_type(self):
+        qs = mock.Mock()
+        value = mock.Mock(
+            start=datetime(2015, 4, 7, 8, 30), stop=datetime(2015, 9, 6, 11, 45))
+        f = DateTimeFromToRangeFilter(lookup_type='gte')
+        f.filter(qs, value)
+        qs.filter.assert_called_once_with(
+            None__range=(datetime(2015, 4, 7, 8, 30), datetime(2015, 9, 6, 11, 45)))
+
+
 class TimeRangeFilterTests(TestCase):
 
     def test_default_field(self):
@@ -734,10 +847,10 @@ class TimeRangeFilterTests(TestCase):
         result = f.filter(qs, None)
         self.assertEqual(qs, result)
 
-    def test_filtering_ignores_lookup_type(self):
+    def test_filtering_ignores_lookup_expr(self):
         qs = mock.Mock()
         value = mock.Mock(start=time(10, 15), stop=time(12, 30))
-        f = TimeRangeFilter(lookup_type='gte')
+        f = TimeRangeFilter(lookup_expr='gte')
         f.filter(qs, value)
         qs.filter.assert_called_once_with(
             None__range=(time(10, 15), time(12, 30)))
@@ -759,3 +872,115 @@ class AllValuesFilterTests(TestCase):
         f.model = mocked
         field = f.field
         self.assertIsInstance(field, forms.ChoiceField)
+
+
+class LookupTypesTests(TestCase):
+    def test_custom_lookup_exprs(self):
+        filters.LOOKUP_TYPES = [
+            ('', '---------'),
+            ('exact', 'Is equal to'),
+            ('not_exact', 'Is not equal to'),
+            ('lt', 'Lesser than'),
+            ('gt', 'Greater than'),
+            ('gte', 'Greater than or equal to'),
+            ('lte', 'Lesser than or equal to'),
+            ('startswith', 'Starts with'),
+            ('endswith', 'Ends with'),
+            ('contains', 'Contains'),
+            ('not_contains', 'Does not contain'),
+        ]
+
+        f = Filter(lookup_expr=None)
+        field = f.field
+        choice_field = field.fields[1]
+        all_choices = choice_field.choices
+
+        self.assertIsInstance(field, LookupTypeField)
+        self.assertEqual(all_choices, filters.LOOKUP_TYPES)
+        self.assertEqual(all_choices[1][0], 'exact')
+        self.assertEqual(all_choices[1][1], 'Is equal to')
+
+        custom_f = Filter(lookup_expr=('endswith', 'not_contains'))
+        custom_field = custom_f.field
+        custom_choice_field = custom_field.fields[1]
+        my_custom_choices = custom_choice_field.choices
+
+        available_lookup_exprs = [
+            ('endswith', 'Ends with'),
+            ('not_contains', 'Does not contain'),
+        ]
+
+        self.assertIsInstance(custom_field, LookupTypeField)
+        self.assertEqual(my_custom_choices, available_lookup_exprs)
+        self.assertEqual(my_custom_choices[0][0], 'endswith')
+        self.assertEqual(my_custom_choices[0][1], 'Ends with')
+        self.assertEqual(my_custom_choices[1][0], 'not_contains')
+        self.assertEqual(my_custom_choices[1][1], 'Does not contain')
+
+
+class CSVFilterTests(TestCase):
+    def setUp(self):
+        class NumberInFilter(BaseCSVFilter, NumberFilter):
+            pass
+
+        class DateTimeYearInFilter(BaseCSVFilter, DateTimeFilter):
+            pass
+
+        self.number_in = NumberInFilter(lookup_expr='in')
+        self.datetimeyear_in = DateTimeYearInFilter(lookup_expr='year__in')
+
+    def test_default_field(self):
+        f = BaseCSVFilter()
+        field = f.field
+        self.assertIsInstance(field, forms.Field)
+
+    def test_concrete_field(self):
+        field = self.number_in.field
+        self.assertIsInstance(field, forms.DecimalField)
+        self.assertIsInstance(field, BaseCSVField)
+        self.assertEqual(field.__class__.__name__, 'DecimalInField')
+
+        field = self.datetimeyear_in.field
+        self.assertIsInstance(field, forms.DateTimeField)
+        self.assertIsInstance(field, BaseCSVField)
+        self.assertEqual(field.__class__.__name__, 'DateTimeYearInField')
+
+    def test_filtering(self):
+        qs = mock.Mock(spec=['filter'])
+        f = self.number_in
+        f.filter(qs, [1, 2])
+        qs.filter.assert_called_once_with(None__in=[1, 2])
+
+    def test_filtering_skipped_with_none_value(self):
+        qs = mock.Mock(spec=['filter'])
+        f = self.number_in
+        result = f.filter(qs, None)
+        self.assertEqual(qs, result)
+
+    def test_field_with_lookup_expr(self):
+        qs = mock.Mock()
+        f = self.datetimeyear_in
+        f.filter(qs, [1, 2])
+        qs.filter.assert_called_once_with(None__year__in=[1, 2])
+
+
+class BaseInFilterTests(TestCase):
+    def test_filtering(self):
+        class NumberInFilter(BaseInFilter, NumberFilter):
+            pass
+
+        qs = mock.Mock(spec=['filter'])
+        f = NumberInFilter()
+        f.filter(qs, [1, 2])
+        qs.filter.assert_called_once_with(None__in=[1, 2])
+
+
+class BaseRangeFilterTests(TestCase):
+    def test_filtering(self):
+        class NumberInFilter(BaseRangeFilter, NumberFilter):
+            pass
+
+        qs = mock.Mock(spec=['filter'])
+        f = NumberInFilter()
+        f.filter(qs, [1, 2])
+        qs.filter.assert_called_once_with(None__range=[1, 2])
