@@ -1,27 +1,31 @@
 from __future__ import absolute_import, unicode_literals
 
 import mock
-import sys
-
-if sys.version_info >= (2, 7):
-    import unittest
-else:  # pragma: nocover
-    from django.utils import unittest  # noqa
+import unittest
 
+import django
+from django.core.exceptions import ValidationError
 from django.db import models
 from django.test import TestCase
 
 from django_filters.filterset import FilterSet
 from django_filters.filterset import FILTER_FOR_DBFIELD_DEFAULTS
-from django_filters.filterset import get_model_field
+from django_filters.filterset import STRICTNESS
+from django_filters.filters import BooleanFilter
 from django_filters.filters import CharFilter
 from django_filters.filters import NumberFilter
 from django_filters.filters import ChoiceFilter
 from django_filters.filters import ModelChoiceFilter
 from django_filters.filters import ModelMultipleChoiceFilter
+from django_filters.filters import UUIDFilter
+from django_filters.filters import BaseInFilter
+from django_filters.filters import BaseRangeFilter
+
+from django_filters.widgets import BooleanWidget
 
 from .models import User
 from .models import AdminUser
+from .models import Article
 from .models import Book
 from .models import Profile
 from .models import Comment
@@ -35,6 +39,7 @@ from .models import DirectedNode
 from .models import Worker
 from .models import HiredWorker
 from .models import Business
+from .models import UUIDTestModel
 
 
 def checkItemsEqual(L1, L2):
@@ -43,20 +48,13 @@ def checkItemsEqual(L1, L2):
     """
     return len(L1) == len(L2) and sorted(L1) == sorted(L2)
 
+
 class HelperMethodsTests(TestCase):
 
     @unittest.skip('todo')
     def test_get_declared_filters(self):
         pass
 
-    def test_get_model_field_none(self):
-        result = get_model_field(User, 'unknown__name')
-        self.assertIsNone(result)
-
-    def test_get_model_field(self):
-        result = get_model_field(Business, 'hiredworker__worker')
-        self.assertEqual(result, HiredWorker._meta.get_field('worker'))
-
     @unittest.skip('todo')
     def test_filters_for_model(self):
         pass
@@ -80,7 +78,7 @@ class DbFieldDefaultFiltersTests(TestCase):
             models.FilePathField,
             models.FloatField,
             models.IntegerField,
-            models.IPAddressField,
+            models.GenericIPAddressField,
             models.NullBooleanField,
             models.PositiveIntegerField,
             models.PositiveSmallIntegerField,
@@ -92,6 +90,7 @@ class DbFieldDefaultFiltersTests(TestCase):
             models.ForeignKey,
             models.OneToOneField,
             models.ManyToManyField,
+            models.UUIDField,
         ]
         msg = "%s expected to be found in FILTER_FOR_DBFIELD_DEFAULTS"
 
@@ -102,7 +101,6 @@ class DbFieldDefaultFiltersTests(TestCase):
         to_check = [
             models.Field,
             models.BigIntegerField,
-            models.GenericIPAddressField,
             models.FileField,
             models.ImageField,
         ]
@@ -120,6 +118,12 @@ class FilterSetFilterForFieldTests(TestCase):
         self.assertIsInstance(result, CharFilter)
         self.assertEqual(result.name, 'username')
 
+    def test_filter_found_for_uuidfield(self):
+        f = UUIDTestModel._meta.get_field('uuid')
+        result = FilterSet.filter_for_field(f, 'uuid')
+        self.assertIsInstance(result, UUIDFilter)
+        self.assertEqual(result.name, 'uuid')
+
     def test_filter_found_for_autofield(self):
         f = User._meta.get_field('id')
         result = FilterSet.filter_for_field(f, 'id')
@@ -175,15 +179,61 @@ class FilterSetFilterForFieldTests(TestCase):
         self.assertIsNotNone(result.extra['queryset'])
         self.assertEqual(result.extra['queryset'].model, Worker)
 
+    @unittest.skipIf(django.VERSION < (1, 9), "version does not support transformed lookup expressions")
+    def test_transformed_lookup_expr(self):
+        f = Comment._meta.get_field('date')
+        result = FilterSet.filter_for_field(f, 'date', 'year__gte')
+        self.assertIsInstance(result, NumberFilter)
+        self.assertEqual(result.name, 'date')
+
     @unittest.skip('todo')
     def test_filter_overrides(self):
         pass
 
 
+class FilterSetFilterForLookupTests(TestCase):
+
+    def test_filter_for_ISNULL_lookup(self):
+        f = Article._meta.get_field('author')
+        result, params = FilterSet.filter_for_lookup(f, 'isnull')
+        self.assertEqual(result, BooleanFilter)
+        self.assertDictEqual(params, {})
+
+    def test_filter_for_IN_lookup(self):
+        f = Article._meta.get_field('author')
+        result, params = FilterSet.filter_for_lookup(f, 'in')
+        self.assertTrue(issubclass(result, ModelChoiceFilter))
+        self.assertTrue(issubclass(result, BaseInFilter))
+        self.assertEqual(params['to_field_name'], 'id')
+
+    def test_filter_for_RANGE_lookup(self):
+        f = Article._meta.get_field('author')
+        result, params = FilterSet.filter_for_lookup(f, 'range')
+        self.assertTrue(issubclass(result, ModelChoiceFilter))
+        self.assertTrue(issubclass(result, BaseRangeFilter))
+        self.assertEqual(params['to_field_name'], 'id')
+
+    def test_isnull_with_filter_overrides(self):
+        class OFilterSet(FilterSet):
+            filter_overrides = {
+                models.BooleanField: {
+                    'filter_class': BooleanFilter,
+                    'extra': lambda f: {
+                        'widget': BooleanWidget,
+                    },
+                },
+            }
+
+        f = Article._meta.get_field('author')
+        result, params = OFilterSet.filter_for_lookup(f, 'isnull')
+        self.assertEqual(result, BooleanFilter)
+        self.assertEqual(params['widget'], BooleanWidget)
+
+
 class FilterSetFilterForReverseFieldTests(TestCase):
 
     def test_reverse_o2o_relationship(self):
-        f = Account._meta.get_field_by_name('profile')[0]
+        f = Account._meta.get_field('profile')
         result = FilterSet.filter_for_reverse_field(f, 'profile')
         self.assertIsInstance(result, ModelChoiceFilter)
         self.assertEqual(result.name, 'profile')
@@ -192,7 +242,7 @@ class FilterSetFilterForReverseFieldTests(TestCase):
         self.assertEqual(result.extra['queryset'].model, Profile)
 
     def test_reverse_fk_relationship(self):
-        f = User._meta.get_field_by_name('comments')[0]
+        f = User._meta.get_field('comments')
         result = FilterSet.filter_for_reverse_field(f, 'comments')
         self.assertIsInstance(result, ModelMultipleChoiceFilter)
         self.assertEqual(result.name, 'comments')
@@ -201,7 +251,7 @@ class FilterSetFilterForReverseFieldTests(TestCase):
         self.assertEqual(result.extra['queryset'].model, Comment)
 
     def test_reverse_m2m_relationship(self):
-        f = Book._meta.get_field_by_name('lovers')[0]
+        f = Book._meta.get_field('lovers')
         result = FilterSet.filter_for_reverse_field(f, 'lovers')
         self.assertIsInstance(result, ModelMultipleChoiceFilter)
         self.assertEqual(result.name, 'lovers')
@@ -210,7 +260,7 @@ class FilterSetFilterForReverseFieldTests(TestCase):
         self.assertEqual(result.extra['queryset'].model, User)
 
     def test_reverse_non_symmetrical_selfref_m2m_field(self):
-        f = DirectedNode._meta.get_field_by_name('inbound_nodes')[0]
+        f = DirectedNode._meta.get_field('inbound_nodes')
         result = FilterSet.filter_for_reverse_field(f, 'inbound_nodes')
         self.assertIsInstance(result, ModelMultipleChoiceFilter)
         self.assertEqual(result.name, 'inbound_nodes')
@@ -219,7 +269,7 @@ class FilterSetFilterForReverseFieldTests(TestCase):
         self.assertEqual(result.extra['queryset'].model, DirectedNode)
 
     def test_reverse_m2m_field_with_through_model(self):
-        f = Worker._meta.get_field_by_name('employers')[0]
+        f = Worker._meta.get_field('employers')
         result = FilterSet.filter_for_reverse_field(f, 'employers')
         self.assertIsInstance(result, ModelMultipleChoiceFilter)
         self.assertEqual(result.name, 'employers')
@@ -322,13 +372,15 @@ class FilterSetClassCreationTests(TestCase):
         self.assertTrue(checkItemsEqual(list(F.base_filters), expected_list))
 
     def test_meta_fields_containing_unknown(self):
-        with self.assertRaises(TypeError):
+        with self.assertRaises(TypeError) as excinfo:
             class F(FilterSet):
                 username = CharFilter()
 
                 class Meta:
                     model = Book
                     fields = ('username', 'price', 'other')
+        self.assertEqual(excinfo.exception.args, (
+            "Meta.fields contains a field that isn't defined on this FilterSet: other",))
 
     def test_meta_fields_dictionary_containing_unknown(self):
         with self.assertRaises(TypeError):
@@ -532,7 +584,21 @@ class FilterSetOrderingTests(TestCase):
 
     def test_ordering_on_unknown_value_results_in_default_ordering_without_strict(self):
         class F(FilterSet):
-            strict = False
+            strict = STRICTNESS.IGNORE
+
+            class Meta:
+                model = User
+                fields = ['username', 'status']
+                order_by = ['status']
+
+        self.assertFalse(F.strict)
+        f = F({'o': 'username'}, queryset=self.qs)
+        self.assertQuerysetEqual(
+            f.qs, ['alex', 'jacob', 'aaron', 'carl'], lambda o: o.username)
+
+    def test_ordering_on_unknown_value_results_in_default_ordering_with_strict_raise(self):
+        class F(FilterSet):
+            strict = STRICTNESS.RAISE_VALIDATION_ERROR
 
             class Meta:
                 model = User
@@ -540,6 +606,14 @@ class FilterSetOrderingTests(TestCase):
                 order_by = ['status']
 
         f = F({'o': 'username'}, queryset=self.qs)
+        with self.assertRaises(ValidationError) as excinfo:
+            f.qs.all()
+        self.assertEqual(excinfo.exception.message_dict,
+                         {'o': ['Select a valid choice. username is not one '
+                                'of the available choices.']})
+
+        # No default order_by should get applied.
+        f = F({}, queryset=self.qs)
         self.assertQuerysetEqual(
             f.qs, ['alex', 'jacob', 'aaron', 'carl'], lambda o: o.username)
 
@@ -558,20 +632,30 @@ class FilterSetOrderingTests(TestCase):
         self.assertQuerysetEqual(
             f.qs, ['carl', 'alex', 'jacob', 'aaron'], lambda o: o.username)
 
-    @unittest.skip('todo')
     def test_ordering_uses_filter_name(self):
         class F(FilterSet):
             account = CharFilter(name='username')
-
             class Meta:
                 model = User
                 fields = ['account', 'status']
                 order_by = True
 
-        f = F({'o': 'username'}, queryset=self.qs)
+        f = F({'o': 'account'}, queryset=self.qs)
         self.assertQuerysetEqual(
             f.qs, ['aaron', 'alex', 'carl', 'jacob'], lambda o: o.username)
 
+    def test_reverted_ordering_uses_filter_name(self):
+        class F(FilterSet):
+            account = CharFilter(name='username')
+            class Meta:
+                model = User
+                fields = ['account', 'status']
+                order_by = True
+
+        f = F({'o': '-account'}, queryset=self.qs)
+        self.assertQuerysetEqual(
+            f.qs, ['jacob', 'carl', 'alex', 'aaron'], lambda o: o.username)
+
     def test_ordering_with_overridden_field_name(self):
         """
         Set the `order_by_field` on the queryset and ensure that the
@@ -642,14 +726,14 @@ class FilterSetTogetherTests(TestCase):
         self.alex = User.objects.create(username='alex', status=1)
         self.jacob = User.objects.create(username='jacob', status=2)
         self.qs = User.objects.all().order_by('id')
-    
+
     def test_fields_set(self):
         class F(FilterSet):
             class Meta:
                 model = User
                 fields = ['username', 'status', 'is_active', 'first_name']
                 together = [
-                    ('username', 'status'), 
+                    ('username', 'status'),
                     ('first_name', 'is_active'),
                 ]
 
@@ -660,14 +744,14 @@ class FilterSetTogetherTests(TestCase):
         f = F({'username': 'alex', 'status': 1}, queryset=self.qs)
         self.assertEqual(f.qs.count(), 1)
         self.assertQuerysetEqual(f.qs, [self.alex.pk], lambda o: o.pk)
-        
+
     def test_single_fields_set(self):
         class F(FilterSet):
             class Meta:
                 model = User
                 fields = ['username', 'status']
                 together = ['username', 'status']
-        
+
         f = F({}, queryset=self.qs)
         self.assertEqual(f.qs.count(), 2)
         f = F({'username': 'alex'}, queryset=self.qs)
